package org.apache.commons.lang3.time;

package .tmp;

import java.text.DateFormat;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.TimeZone;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import org.apache.commons.lang3.LocaleUtils;
import org.apache.commons.lang3.Validate;

abstract class FormatCache<F extends Format> {
  static final int NONE = -1;
  
  private final ConcurrentMap<ArrayKey, F> cInstanceCache = new ConcurrentHashMap<>(7);
  
  private static final ConcurrentMap<ArrayKey, String> cDateTimeInstanceCache = new ConcurrentHashMap<>(7);
  
  public F getInstance() {
    return getDateTimeInstance(3, 3, TimeZone.getDefault(), Locale.getDefault());
  }
  
  public F getInstance(String pattern, TimeZone timeZone, Locale locale) {
    Validate.notNull(pattern, "pattern", new Object[0]);
    if (timeZone == null)
      timeZone = TimeZone.getDefault(); 
    locale = LocaleUtils.toLocale(locale);
    ArrayKey key = new ArrayKey(new Object[] { pattern, timeZone, locale });
    Format format = (Format)this.cInstanceCache.get(key);
    if (format == null) {
      format = (Format)createInstance(pattern, timeZone, locale);
      Format format1 = (Format)this.cInstanceCache.putIfAbsent(key, (F)format);
      if (format1 != null)
        format = format1; 
    } 
    return (F)format;
  }
  
  protected abstract F createInstance(String paramString, TimeZone paramTimeZone, Locale paramLocale);
  
  private F getDateTimeInstance(Integer dateStyle, Integer timeStyle, TimeZone timeZone, Locale locale) {
    locale = LocaleUtils.toLocale(locale);
    String pattern = getPatternForStyle(dateStyle, timeStyle, locale);
    return getInstance(pattern, timeZone, locale);
  }
  
  F getDateTimeInstance(int dateStyle, int timeStyle, TimeZone timeZone, Locale locale) {
    return getDateTimeInstance(Integer.valueOf(dateStyle), Integer.valueOf(timeStyle), timeZone, locale);
  }
  
  F getDateInstance(int dateStyle, TimeZone timeZone, Locale locale) {
    return getDateTimeInstance(Integer.valueOf(dateStyle), null, timeZone, locale);
  }
  
  F getTimeInstance(int timeStyle, TimeZone timeZone, Locale locale) {
    return getDateTimeInstance(null, Integer.valueOf(timeStyle), timeZone, locale);
  }
  
  static String getPatternForStyle(Integer dateStyle, Integer timeStyle, Locale locale) {
    Locale safeLocale = LocaleUtils.toLocale(locale);
    ArrayKey key = new ArrayKey(new Object[] { dateStyle, timeStyle, safeLocale });
    String pattern = cDateTimeInstanceCache.get(key);
    if (pattern == null)
      try {
        DateFormat formatter;
        if (dateStyle == null) {
          formatter = DateFormat.getTimeInstance(timeStyle.intValue(), safeLocale);
        } else if (timeStyle == null) {
          formatter = DateFormat.getDateInstance(dateStyle.intValue(), safeLocale);
        } else {
          formatter = DateFormat.getDateTimeInstance(dateStyle.intValue(), timeStyle.intValue(), safeLocale);
        } 
        pattern = ((SimpleDateFormat)formatter).toPattern();
        String previous = cDateTimeInstanceCache.putIfAbsent(key, pattern);
        if (previous != null)
          pattern = previous; 
      } catch (ClassCastException ex) {
        throw new IllegalArgumentException("No date time pattern for locale: " + safeLocale);
      }  
    return pattern;
  }
}
